#ifndef AIRSIM_CLIENT_COMMON_TYPES
#define AIRSIM_CLIENT_COMMON_TYPES

#include <common/VectorMath.hpp>
#include <common/CommonStructs.hpp>
#include <common/ImageCaptureBase.hpp>

#include <chrono>
#include <string>

namespace airsim_client {
	using namespace std::chrono_literals;

	using TimeStamp = std::chrono::time_point<std::chrono::steady_clock, std::chrono::duration<double>>;

	struct Image
	{
		explicit Image(msr::airlib::ImageCaptureBase::ImageResponse r)
			: 	time_stamp{std::chrono::duration<double, std::nano>{r.time_stamp}},
				height{r.height},
				width{r.width},
				image_type{r.image_type},
				pixels_as_float{r.pixels_as_float},
				image_data_uint8{std::move(r.image_data_uint8)},
				image_data_float{std::move(r.image_data_float)} {}

		TimeStamp time_stamp;
		int height;
		int width;
		msr::airlib::ImageCaptureBase::ImageType image_type;
		bool pixels_as_float;

		std::vector<uint8_t> image_data_uint8;
		std::vector<float> image_data_float;
	};

	struct CameraInfo
	{
		std::string optical_frame_id;
		TimeStamp time_stamp;
		int image_width = 0;
		int image_height = 0;

		int image_type = -1;

		msr::airlib::VectorMathf::Matrix3x3f intrinsic_matrix;
		//Camera position in the optical frame described in the optical_fram_id.
		msr::airlib::VectorMathd::Vector3f position;
	};

	struct Transform
	{
		std::string parent_frame_id;
		std::string child_frame_id;
		TimeStamp time_stamp;
		msr::airlib::Pose transform;
	};

} // namespace airsim_client

#endif // AIRSIM_CLIENT_COMMON_TYPES
