#ifndef AIRSIM_CLIENT_RPC_CLIENT
#define AIRSIM_CLIENT_RPC_CLIENT

#include <rpc/msgpack.hpp>
#ifndef RPCLIB_MSGPACK
#define RPCLIB_MSGPACK clmdep_msgpack
#endif // !RPCLIB_MSGPACK
#include <rpc/client.h>

#include <common/ImageCaptureBase.hpp>
#include <sensors/imu/ImuBase.hpp>
#include <vehicles/multirotor/api/MultirotorCommon.hpp>
#include <string>

namespace airsim_client {

	
	
	class RpcClient {
	
	public:

		explicit RpcClient(const std::string& ip = "127.0.0.1", uint16_t port = 41451, float timeout_sec = 1.0f);
	
		void reset();
		void waitForConnection();
		void disconnect();
	
		rpc::client::connection_state getConnectedState();
	
		bool armDisarm(bool arm, const std::string& vehicle_name = "");
		void enableApiControl(bool enable, const std::string& vehicle_name = "");
		bool isApiControlEnabled(const std::string& vehicle_name = "");

		void simContinueForTime(double sec);
		void simPause(bool pause);
		bool isSimPaused();

		RpcClient* takeoffAsync(float timeout_sec = 20, const std::string& vehicle_name = "");
		RpcClient* landAsync(float timeout_sec = 60, const std::string& vehicle_name = "");
		RpcClient* hoverAsync(const std::string& vehicle_name = "");
		RpcClient* moveByVelocityAsync(msr::airlib::VectorMathf::Vector3f velocity, float duration,
			msr::airlib::DrivetrainType drivetrain,
			const msr::airlib::YawMode& yaw_mode,
			const std::string& vehicle_name = "");

		using AirSimImage = msr::airlib::ImageCaptureBase;
		using AirSimImu = msr::airlib::ImuBase;

		std::vector<AirSimImage::ImageResponse> simGetImages(const std::vector<AirSimImage::ImageRequest>& requests, const std::string& vehicle_name = "");
		AirSimImu::Output getImuData(const std::string& imu_name = "", const std::string& vehicle_name = "");
	
		void cancelLastTask(const std::string& vehicle_name = "");
	
		void sendLogMessage(const std::string& msg, const std::string& msg_param = "", unsigned int log_level = 0);
	
		bool waitOnLastTask(float timeout_sec = 60.0f);

	protected:

		rpc::client client_;
		std::future<RPCLIB_MSGPACK::object_handle> last_future_;
	};

}//namespace airsim_client
#endif // AIRSIM_CLIENT_RPC_CLIENT
