#ifndef AIRSIM_CLIENT_SETTINGS_PARSER
#define AIRSIM_CLIENT_SETTINGS_PARSER

#include <common/AirSimSettings.hpp>

namespace airsim_client {

	// Singleton class made to initialize AirSimSettings correctly
	// Currently only for VehicleSettings and simmode name
	class SettingsParser {

		using AirSimSettings = msr::airlib::AirSimSettings;
		using IType = msr::airlib::ImageCaptureBase::ImageType;

	// Static member functions
	public:
		inline static SettingsParser& Get() 
		{
			// Constructor thows a std::runtime_error if it is unable
			// read the settings file.
			static SettingsParser instance;
			return instance; 
		}

		static constexpr const char* getCaptureTypeName(int type) noexcept {
			switch (type) {
				case 0: return "scene";
				case 1: return "depth_planner";
				case 2: return "depth_perspective";
				case 3: return "depth_vis";
				case 4: return "disparity_normalized";
				case 5: return "segmentation";
				case 6: return "surface_normals";
				case 7: return "infrared";
				default: return "unknown"; // This should not happen
			}
		}

		static constexpr int getCaptureType(IType type) noexcept {
			switch (type) {
				case IType::Scene: return 0;
				case IType::DepthPlanner: return 1;
				case IType::DepthPerspective: return 2;
				case IType::DepthVis: return 3;
				case IType::DisparityNormalized: return 4;
				case IType::Segmentation: return 5;
				case IType::SurfaceNormals: return 6;
				case IType::Infrared: return 7;
				default: return -1; // This should not happen
			}
		}

	// Member functions, accessible through SettingsParser::Get()
	public:

		inline std::string SimMode() const { return AirSimSettings::singleton().simmode_name; }
		inline std::map<std::string, std::unique_ptr<AirSimSettings::VehicleSetting>>& VehicleMap()
		{
			return AirSimSettings::singleton().vehicles;
		}

	protected:
		SettingsParser();
	};

} // namespace airsim_client
#endif // AIRSIM_CLIENT_SETTINGS_PARSER
