#ifndef AIRSIM_CLIENT_VEHICLE
#define AIRSIM_CLIENT_VEHICLE

#include "AirSimTools/CommonTypes.hpp"

#include <common/AirSimSettings.hpp>

#include <map>
#include <string>
#include <vector>

namespace airsim_client {


	class Vehicle {

		using AirSimSettings = msr::airlib::AirSimSettings;
		using Kinematics = msr::airlib::Kinematics;

	public:

		Vehicle(std::string map_frame_name, AirSimSettings::VehicleSetting& settings);

		inline std::string getName() const { return vehicle_name_; }
		inline std::vector<Transform> getStaticTransforms() const { return static_transforms_; }
		inline Transform getCurrentTransform() const { return current_transform_; }
		inline std::unordered_map<int, CameraInfo> getCameraInfos() const { return camera_infos_; }

		void updateCurrentTransform(const Kinematics::State& kinematics, TimeStamp time_stamp);

	private:
		void addSpawnPose(const AirSimSettings::VehicleSetting& settings);
		void addCameraTransform(const std::string& camera_name, const AirSimSettings::CameraSetting& settings);
		void addCameraInfo(const std::string& camera_name, const AirSimSettings::CaptureSetting& capture_setting);
		void convertNanToZeroInPose(msr::airlib::Vector3r& position, AirSimSettings::Rotation& rotation);

	public:
		msr::airlib::VectorMathf::Vector3f vel_body_{0,0,0};
		msr::airlib::VectorMathf::Vector3f vel_world_{0,0,0};
		float speed_ = 2; // m/s

	private:
		std::string map_frame_name_;
		std::string vehicle_name_;

		std::unordered_map<int, CameraInfo> camera_infos_;

		// Current transforms
		Transform current_transform_;

		// Static transforms
		Transform spawn_pose_;
		std::vector<Transform> static_transforms_;
	};

} // namespace airsim_client

#endif //AIRSIM_CLIENT_TRANSFORMS
