#ifndef AIRSIM_CLIENT
#define AIRSIM_CLIENT

#include "Node.hpp"
#include "Events/ClientEvent.hpp"
#include "Window.hpp"

#include <memory>
#include <vector>
#include <set>
#include <unordered_map>
#include <chrono>

namespace airsim_client { namespace core {

	class Client {

	public:
		Client();

		void AddNode(std::unique_ptr<Node> node);
		void Run();
		void OnEvent(Event& e);

		inline Window* GetWindow() { return window_.get(); }

	private:
		void StartNode(Node* node);
		void StopNode(Node* node);

		bool OnShutdownEvent();
		bool OnNodeNotFoundEvent();
		bool OnWindowCloseEvent();
		bool OnWindowResizeEvent(const WindowResizeEvent& e);

	private:

		std::unique_ptr<Window> window_;

		std::vector<std::unique_ptr<Node>> all_nodes_;
		std::vector<Node*> active_nodes_;

		bool running_ = true;
	};

}} //namespace airsim_client::core
#endif // AIRSIM_CLIENT
