#include "Core/Event.hpp"

#include <string>
#include <sstream>

namespace airsim_client { namespace core {

	class KeyEvent : public Event {

	public:
		EVENT_CLASS_CATEGORY(EventCategoryKeyboard)

		inline int GetKeyCode() const { return key_code_; }

	protected:
		explicit KeyEvent(int keycode)
			: key_code_(keycode) {}

		int key_code_;
	};

	class KeyPressedEvent : public KeyEvent {
		
	public:	
		EVENT_CLASS_TYPE(EventType::KeyPressed)

		explicit KeyPressedEvent(int keycode, bool repeated = false)
			: KeyEvent{keycode}, repeated_{repeated} {}

		inline bool IsRepeated() const { return repeated_; }

		std::string ToString() const override {
			std::stringstream ss;
			ss 	<< "KeyPressedEvent: " << key_code_ 
				<< " , repeated = " << (repeated_ ? "true" : "false") << std::endl;
			return ss.str();
		}

	private:
		bool repeated_ = false;
	};

	class KeyReleasedEvent : public KeyEvent {

	public:
		EVENT_CLASS_TYPE(EventType::KeyReleased)

		explicit KeyReleasedEvent(int keycode)
			: KeyEvent(keycode) {}

		std::string ToString() const override {
			std::stringstream ss;
			ss 	<< "KeyReleasedEvent: " << key_code_  << std::endl;
			return ss.str();
		}
	};

}} //namespace airsim_client::core
