#ifndef AIRSIM_CLIENT_INPUT
#define AIRSIM_CLIENT_INPUT

#include <GLFW/glfw3.h>
#include <utility>

namespace airsim_client { namespace core {

	class Input {
	public:
		constexpr static void SetInputWindow(GLFWwindow* window) { Input::s_window_ = window; }

		inline static bool IsKeyPressed(int keycode)
		{
			auto state = glfwGetKey(s_window_, keycode);
			return state == GLFW_PRESS || state == GLFW_REPEAT;
		}

		inline static bool IsMouseButtonPressed(int button)
		{
			auto state = glfwGetMouseButton(s_window_, button);
			return state == GLFW_PRESS;
		}

		inline static std::pair<double, double> GetMouseXY()
		{
			double x_pos = 0.0,	y_pos = 0.0;
			glfwGetCursorPos(s_window_, &x_pos, &y_pos);
			return {x_pos, y_pos};
		}

	private:
		static GLFWwindow* s_window_;
	};

}} // namespace airsim_client::core
#endif // AIRSIM_CLIENT_INPUT
