#ifndef AIRSIM_CLIENT_LOGGER
#define AIRSIM_CLIENT_LOGGER

#include <spdlog/spdlog.h>
#include <spdlog/fmt/ostr.h>

namespace airsim_client { namespace core {

	class Log {

	public:
		static void Init();

		inline static std::shared_ptr<spdlog::logger>& GetLogger() { return logger_; }
		inline static std::shared_ptr<spdlog::logger>& GetFileLogger() { return file_logger_; }

	private:
		static std::shared_ptr<spdlog::logger> logger_;
		static std::shared_ptr<spdlog::logger> file_logger_;

	};

} // namespace core
} // namespace airsim_client

/*-------------------------------------------------------------
				STDOUT LOGGING MACROS
-------------------------------------------------------------*/
#define CLIENT_ERROR(...)	::airsim_client::core::Log::GetLogger()->error(__VA_ARGS__);
#define CLIENT_WARN(...) 	::airsim_client::core::Log::GetLogger()->warn(__VA_ARGS__);
#define CLIENT_INFO(...) 	::airsim_client::core::Log::GetLogger()->info(__VA_ARGS__);
#define CLIENT_DEBUG(...) 	::airsim_client::core::Log::GetLogger()->debug(__VA_ARGS__);

#define CLIENT_FATAL(...) 	::airsim_client::core::Log::GetLogger()->error(__VA_ARGS__);\
							throw(std::runtime_error{__VA_ARGS__});

#ifdef AIRSIM_CLIENT_DEBUG
#define CLIENT_EVENT(...)	::airsim_client::core::Log::GetLogger()->warn(__VA_ARGS__);
#else
#define CLIENT_EVENT(...)
#endif // AIRSIM_CLIENT_DEBUG

							
/*-------------------------------------------------------------
				FILE LOGGING MACROS
-------------------------------------------------------------*/
#define FILE_CLIENT_ERROR(...)	::airsim_client::core::Log::GetFileLogger()->error(__VA_ARGS__);
#define FILE_CLIENT_WARN(...) 	::airsim_client::core::Log::GetFileLogger()->warn(__VA_ARGS__);
#define FILE_CLIENT_INFO(...) 	::airsim_client::core::Log::GetFileLogger()->info(__VA_ARGS__);
#define FILE_CLIENT_DEBUG(...)	::airsim_client::core::Log::GetFileLogger()->debug(__VA_ARGS__);

#define FILE_CLIENT_FATAL(...) 	::airsim_client::core::Log::GetFileLogger()->error(__VA_ARGS__);\
							throw(std::runtime_error{__VA_ARGS__});

#ifdef AIRSIM_CLIENT_DEBUG
#define FILE_CLIENT_EVENT(...)	::airsim_client::core::Log::GetFileLogger()->warn(__VA_ARGS__);
#else
#define FILE_CLIENT_EVENT(...)
#endif // AIRSIM_CLIENT_DEBUG

#endif // AIRSIM_CLIENT_LOGGER
