#ifndef AIRSIM_WINDOW
#define AIRSIM_WINDOW

#include "Event.hpp"

struct GLFWwindow;

namespace airsim_client { namespace core {

	struct WindowProperties {

		const std::string title = "Press Escape to exit";
		int width = 720;
		int height = 720; 

	};

	class Window {

	public:
		explicit Window(const WindowProperties& props = WindowProperties());
		~Window() noexcept;
		Window(const Window&) = delete;
		Window(Window&&) noexcept = delete;
		Window& operator = (const Window&) = delete;
		Window& operator = (Window&&) noexcept = delete;

		void OnUpdate();

		inline unsigned int GetHeight() const { return data_.height; }
		inline unsigned int GetWidth() const { return data_.width; }

		inline GLFWwindow* GetGLFWwindowPtr() const { return window_; }

		using EventCallbackFn = std::function<void (Event&)>;
		inline void SetEventCallback(const EventCallbackFn& callback) { data_.EventCallback = callback; }

	private:
		GLFWwindow* window_;

		struct WindowData {

			explicit WindowData(const WindowProperties& props)
				: title{props.title}, width{props.width}, height{props.height} {} 
			std::string title;
			int width;
			int height;

			EventCallbackFn EventCallback;
		};

		WindowData data_;
	};

}} // namespace airsim_client::core


#endif // AIRSIM_WINDOW
