#ifndef AIRSIM_CLIENT_CV_NODE
#define AIRSIM_CLIENT_CV_NODE

#include "Core/Node.hpp"
#include "Events/AirSimEvent.hpp"
#include "AirSimTools/RpcClient.hpp"
#include "AirSimTools/Vehicle.hpp"

#include <common/ImageCaptureBase.hpp>

#include <memory>

namespace airsim_client {

	class CvNode : public core::Node {

	public:
		CvNode();
		~CvNode() noexcept final;

		// Should be held and managed by the Client for the whole program duration
		CvNode(const CvNode&) = delete;
		CvNode& operator = (const CvNode&) = delete;
		CvNode(CvNode&&) noexcept = delete;
		CvNode& operator = (CvNode&&) noexcept = delete;

	public:
		void OnStartup() final;
		void OnShutdown() final;
		void OnUpdate(Seconds dt) final;
		void OnEvent(core::Event& e) final;

		inline void SetEventCallbackFn(const EventCallbackFn& callback) final { EventCallback = callback; }

	private:

		void initializeVehicleSettings();

	private:

		std::unique_ptr<RpcClient> Client;
		EventCallbackFn EventCallback;

		using ImageRequest = msr::airlib::ImageCaptureBase::ImageRequest;
		std::vector<ImageRequest> requests_;

		std::unique_ptr<Vehicle> vehicle_ ;
	};

} //namespace airsim_client
#endif //AIRSIM_CLIENT_AIRSIM_NODE
