#ifndef AIRSIM_CLIENT_MULTIROTOR_NODE
#define AIRSIM_CLIENT_MULTIROTOR_NODE

#include "Core/Node.hpp"
#include "Core/Events/KeyEvent.hpp"
#include "Events/AirSimEvent.hpp"
#include "AirSimTools/Vehicle.hpp"

#include <vehicles/multirotor/api/MultirotorRpcLibClient.hpp>

#include <common/CommonStructs.hpp>
#include <common/ImageCaptureBase.hpp>

#include <memory>
#include <set>
#include <chrono>

namespace airsim_client {

	using namespace std::chrono_literals;

	class MultirotorNode : public core::Node {

	public:
		explicit MultirotorNode(Milliseconds update_interval);
		~MultirotorNode() noexcept final;

		// Should be held and managed by the Client for the whole program duration
		MultirotorNode(const MultirotorNode&) = delete;
		MultirotorNode& operator = (const MultirotorNode&) = delete;
		MultirotorNode(MultirotorNode&&) noexcept = delete;
		MultirotorNode& operator = (MultirotorNode&&) noexcept = delete;

	public:
		void OnStartup() final;
		void OnShutdown() final;
		void OnUpdate(Seconds dt) final;
		void OnEvent(core::Event& e) final;

		inline void SetEventCallbackFn(const EventCallbackFn& callback) final { EventCallback = callback; }

	private:

		void initializeVehicleSettings();
		void updateAndSendTransform(Vehicle& vehicle) const;
		void setVelocity(Vehicle& vehicle) const;
		void receiveAndSendImages(const Vehicle& vehicle) const;
		void receiveAndSendSensorData(const Vehicle& vehicle) const;

		constexpr bool isConnected() const noexcept {
			using State = msr::airlib::RpcLibClientBase::ConnectionState;
			return Client->getConnectionState() == State::Connected;
		}

	private:

		std::unique_ptr<msr::airlib::MultirotorRpcLibClient> Client;
		EventCallbackFn EventCallback;

		using ImageRequest = msr::airlib::ImageCaptureBase::ImageRequest;
		std::vector<ImageRequest> requests_;

		std::vector<Vehicle> multirotors_;
		std::vector<Transform> static_tf_msgs_;
		bool static_tf_msgs_updated_ = false;

		Milliseconds accumulated_time_ = 0ms;
		const Milliseconds update_interval_  = 200ms;
	};

} //namespace airsim_client
#endif //AIRSIM_CLIENT_AIRSIM_NODE
