#include "OdometryErrorNode.hpp"

#include <Eigen/Core>

namespace airsim_client {

	void OdometryErrorNode::OnStartup() {

	}

	void OdometryErrorNode::OnShutdown() {
		CLIENT_WARN("Mean: {0}", tf_mse_)
	}

	void OdometryErrorNode::OnUpdate([[gnu::unused]] Seconds dt) {
		UpdatePositionError();
	}

	void OdometryErrorNode::OnEvent(core::Event &e) {
		core::EventDispatcher dispatcher(e);
		dispatcher.Dispatch<TransformPairEvent>([this](const TransformPairEvent& e) {
			return this->OnTransformPairEvent(e);
		});
	}

	bool OdometryErrorNode::OnTransformPairEvent(const TransformPairEvent& e) {
		tf_queue_.push(e.GetPair());
		return true;
	}

	void OdometryErrorNode::UpdatePositionError() {
		while(!tf_queue_.empty()) {
			auto [first, second] = std::move(tf_queue_.front());
			tf_queue_.pop();

			Eigen::Vector3f estimate {
				second.transform.translation.x,
				second.transform.translation.y,
				second.transform.translation.z
			};

			Eigen::Vector3f ground_truth{
				first.transform.translation.x,
				first.transform.translation.y,
				first.transform.translation.z,
			};

			auto error = (estimate - ground_truth);
			tf_mse_ = (tf_mse_ * num_measurements_ + error.transpose() * error)/static_cast<float>(num_measurements_ + 1);
			num_measurements_++;
		}
	}

} // namespace airsim_client
